/* Our math.h for HAS_FPCLASSIFY in plan9.config
 *           FP_NORMAL     Normalized
 *           FP_ZERO       Zero
 *           FP_INFINITE   Infinity
 *           FP_SUBNORMAL  Denormalized
 *           FP_NAN        NaN
 */
#define FP_NORMAL 1
#define FP_ZERO 2
#define FP_INFINITE 3
#define FP_SUBNORMAL 4
#define FP_NAN 5
